#include "BMPLoader.h"

// getint and getshort are helper functions to load larger data types
// in Big Endian CPUs such as those in older Mac (PowerPC) or Solaris
// (SPARC) workstations. They are needed because BMP files are
// designed for Little Endian CPUs like the Intel x86 series.
//
// Originally from the Xv bmp loader.

// Ensure that Little Endian ints are read into memory correctly on
// Big Endian platforms
static unsigned int getint(FILE *fp)
{
	unsigned int c, c1, c2, c3;

	// get 4 bytes
	c  = ((unsigned int)getc(fp));  
	c1 = ((unsigned int)getc(fp)) << 8;  
	c2 = ((unsigned int)getc(fp)) << 16;  
	c3 = ((unsigned int)getc(fp)) << 24;  

	return c | c1 | c2 | c3;
}


// Ensure that Little Endian shorts are read into memory correctly on
// Big Endian platforms
static unsigned short getshort(FILE* fp)
{
	unsigned short c, c1;

	//get 2 bytes
	c  = ((unsigned short)getc(fp));  
	c1 = ((unsigned short)getc(fp)) << 8;  

	return c | c1;
}

// quick and dirty bitmap loader...for 24 bit bitmaps with 1 plane only.  
// See http://www.dcs.ed.ac.uk/~mxr/gfx/2d/BMP.txt for more info.

bool ImageLoad(const char *filename, Image *image)
{
	FILE *file;
	unsigned long size;          // size of the image in bytes.
	size_t i,j,k, linediff;		 // standard counter.
	unsigned short int planes;   // number of planes in image (must be 1) 
	unsigned short int bpp;      // number of bits per pixel (must be 24)
	char temp;                   // temporary storage for bgr-rgb conversion.

	// make sure the file is there.
	if ((file = fopen(filename, "rb"))==NULL) 
	{
		printf("File Not Found : %s\n",filename);
		return false;
	}

	// seek through the bmp header, up to the width/height:
	fseek(file, 18, SEEK_CUR);

	// read the width
	image->sizeX = getint (file);
	printf("Width of %s: %d\n", filename, image->sizeX);

	// read the height 
	image->sizeY = getint (file);
	printf("Height of %s: %d\n", filename, image->sizeY);

	// calculate the size (assuming 24 bits or 3 bytes per pixel).
	// BMP lines are padded to the nearest double word boundary.
	// fortunat
	size = GLsizei(4.0*ceil(image->sizeX*3.0/4.0) * image->sizeY) ;

	// read the planes
	planes = getshort(file);
	if (planes != 1) 
	{
		printf("Planes from %s is not 1: %u\n", filename, planes);
		return false;
	}

	// read the bpp
	bpp = getshort(file);
	if (bpp != 24) 
	{
		printf("Bpp from %s is not 24: %u\n", filename, bpp);
		return 0;
	}

	// seek past the rest of the bitmap header.
	fseek(file, 24, SEEK_CUR);

	// allocate space for the data. 
	image->data = new GLubyte[size];
	if (image->data == NULL) 
	{
		printf("Error allocating memory for color-corrected image data");
		return false;
	}

	// read the data
	i = fread(image->data, size, 1, file);
	if (i != 1) 
	{
		printf("Error reading image data from %s.\n", filename);
		return false;
	}


	// reverse all of the colors. (bgr -> rgb)
	// calculate distance to 4 byte boundary for each line.
	// if this distance is not 0, then there will be a color reversal error
	// unless we correct for the distance on each line.
	linediff = size_t(4.0*ceil(image->sizeX*3.0/4.0) - image->sizeX*3.0);
	k = 0;
	for (j=0;j<image->sizeY;j++)
	{

		for (i=0;i<image->sizeX;i++) 
		{
			temp = image->data[k];
			image->data[k] = image->data[k+2];
			image->data[k+2] = temp;
			k+=3;
		}
		k+= linediff;

	}


	// we're done.
	return true;
}
